#include "BF537 Flags.h"

//--------------------------------------------------------------------------//
// Function:	Init_Flags											
//																		
// Parameters:	None													
//																	
// Return:		None												
//																	
// Description:	This function configures PORTF2 as input for edge sensitive
//				interrupt generation.					
//
//--------------------------------------------------------------------------//
void Init_Flags(void)
{
    int temp;
    	
	temp = *pPORTF_FER;
	temp++;
#if (__SILICON_REVISION__ < 0x0001)
	*pPORTF_FER = 0x80C0;
	*pPORTF_FER = 0x80C0;
#else
	*pPORTF_FER = 0x80C0;               //enable TMR2, TMR3, and TMRCLK functions
                                        //GPIO registers settings irrelevent
                                        //for these signals
#endif

	*pPORTFIO_INEN		= 0x8124;       //GPIO inputs PF8, PF5, PF2,and TMRCLK
	*pPORTFIO_DIR		= 0x0018;       //PF3 and PF4 are outputs
	*pPORTFIO_POLAR     = 0x0100;       //PF8 will be falling edge interrupt for AD7656 BUSY
	*pPORTFIO_EDGE		= 0x0124;       //GPIO inputs PF8, PF5, PF2 edge sensitive interrupts
	*pPORTFIO_MASKA		= 0x0024;       //enable interrupts for mode buttons (IRQ27)
	*pPORTFIO_MASKB		= 0x0100;       //enable interrupts for AD7656 BUSY signal (IRQ31)       
}

//--------------------------------------------------------------------------//
// Function:	Init_Timers								
//																	
// Parameters:	None												
//																	
// Return:		None													
//																	
// Description:	This function initialises Timer2 and Timer3 for PWM mode.
//				                                               			
//--------------------------------------------------------------------------//

void Init_Timers(void)
{
	*pTIMER3_CONFIG		= 0x000D;       //PWM mode, SCLK based, no IRQ
	*pTIMER3_PERIOD		= 0x0000000A;   //sclk = 100 MHz
	*pTIMER3_WIDTH		= 0x00000005;   //divide by 10
	
    //*pTIMER3_PERIOD		= 0x000000014;   //sclk = 100 MHz
    //*pTIMER3_WIDTH		= 0x0000000A;    //divide by 10
	
	
    //*pTIMER2_CONFIG		= 0x00BD;    //PWM mode, TMRCLK based, IRQ for interim test only                                    
	
    //other TIMER2 related code remains for debug, but the next line disables the IRQ
	
	*pTIMER2_CONFIG		= 0x00AD;       //PWM mode, TMRCLK based, drives AD7656 CONVST
	*pTIMER2_PERIOD		= 0x000000A2;   //TMRCLK =  10 MHz
	*pTIMER2_WIDTH		= 0x0000009F;   //159 count high - 3 counts low (AD7656 CONVSTA)
	
	*pTIMER_ENABLE		= 0x000C;       //enable TIMERS 2 and 3
}

//--------------------------------------------------------------------------//
// Function:	Init_Interrupts											
//																
// Parameters:	None												
//																		
// Return:		None												
//																	
// Description:	This function initializes the interrupts for Timer2	    
//				PORTF_IntA, and PORTF_IntB 							
//--------------------------------------------------------------------------//

void Init_Interrupts(void)
{
	// assign core IDs to interrupts
	*pSIC_IAR0 = 0xffffffff;
	*pSIC_IAR1 = 0xffffffff;
	*pSIC_IAR2 = 0xff4fffff;					// Timer2     -> ID4; 
	*pSIC_IAR3 = 0x6f7f5fff;					// PORTF IntA -> ID5, PORTF IntB -> ID6
                                                // Mem0Stream -> ID7
                                                // IDx associated with IVG level so
                                                // ID6 implies IVG13 from table 4-3 of
                                                // Hardware Reference Manual (HRM)

    // assign ISRs to interrupt vectors
	register_handler(ik_ivg11, Timer2_ISR);         // Timer2 ISR -> IVG 11
	register_handler(ik_ivg12, PORTF_IntA_ISR);     // PORTF_IntA_ISR -> IVG12 (PF2, PF5)
	register_handler(ik_ivg13, PORTF_IntB_ISR);     // PORTF_IntB_ISR -> IVG13 (AD7656 BUSY/PF8)
    register_handler(ik_ivg14, Mem0StreamDMA_ISR);  // Mem0Stream_ISR -> IVG14
	
	// enable Timer2, memDMA0, and PORTF IntA and IntB interrupts
	*pSIC_IMASK = 0xA8200000;
}

//--------------------------------------------------------------------------//
// Function:	Init_EBIU						         				
//																	
// Parameters:	None												
//																	
// Return:		None												
//																		
// Description:	This function initialises the external bus controller    
//				                        								
//--------------------------------------------------------------------------//
void Init_EBIU(void)
{
                                                     //enable async banks 0 and 1
    *pEBIU_AMBCTL0 = 0xAAC2FFC2;                     //configure bank 1 for AD7656 access
    *pEBIU_AMGCTL  = 0x01F4;                         //bank 0 default settings - assumes
                                                     //flash for boot only
                                                     
    // AMS1 read of AD7656  - 10 cycles for now. Can probably go to 6 or 7.
    // so w/SCLK=10MHz: 10*10*6 = 600 ns to read 6 channels.
    // 61728 Hz CONVST rate allows for 8100 cclk between samples; so, without ping pong 
    // buffer scheme, 7500 cclks available for FFT.                                                
                                                     
    // ADSP-BF537-proc.xml reset values - SDRAM initialization
    
    *pEBIU_SDRRC   = 0x03A0;         //SDRAM Refresh Rate Control Register
    *pEBIU_SDBCTL  = 0x0025;         //SDRAM Memory Bank Control Register
    *pEBIU_SDGCTL  = 0x0091998D;     //SDRAM Memory Global Control Register
    
}

//--------------------------------------------------------------------------//
// Function:	Init_AD7656						         			
//																
// Parameters:	None											
//																
// Return:		None											
//																
// Description:	This function initialises the AD7656 ADC       
//				                        						
//--------------------------------------------------------------------------//
void Init_AD7656(void)
{
    
    int i, j, k;
    	
    //reset AD7656
    
	*pPORTFIO_SET 		= 0x0010;       // drive AD7656 RESET (active high) 
	                                    // AD7656 H/S_ driven low   
    
	//asm("r6=cycles;");                //debug support
	
	for(i=0; i<1000; i++)               //delay 13,028,017 cycles
	     {                              //or 26ms for 500 MHz cclk
	         for(j=0; j<1000; j++) 
	             { 
	                 asm("nop;");
	             }
	     }
	     
	
	//asm("r7=cycles;");                 //debug support
	//asm("r5=r7-r6;");                  //debug support
	
	
	*pPORTFIO_CLEAR		= 0x0010;        //clear AD7656 RESET    

	

	
	for(i=0; i<10000; i++)               //delay (13,028,017 * 10) cycles
	     {                               //or 260ms for 500 MHz cclk
	         for(j=0; j<1000; j++)       //-revisit this delay which can likely be reduced
	             {                       //-by factor of ten minimum
	                 asm("nop;");
	             }
	     } 
	        

	     
		     
    //Initialize AD7656 control register
	    
	    
	for(k=0; k<5; k++)
	{         
    	*pAD7656 = AD7656DATA;               //see BF537_Flags.h 
   
    	for(i=0; i<10; i++)                  //delay
	     {                            
	        for(j=0; j<100; j++) 
	             { 
	                 asm("nop;");
	             }
	     }
	 }	     
    
}

